      SUBROUTINE PCMATCHK( ICONTXT, M, N, MB, NB, A, LDA, B, LDB, IAROW,
     $                     IACOL, EPS, ERROR )
*
*  -- PB-BLAS auxiliary routine (version 2.0) --
*     Oak Ridge National Laboratory and University of Tennessee
*     February 28, 1995
*
*     ..
*     .. Scalar Arguments ..
      INTEGER              ICONTXT, M, N, MB, NB, LDA, LDB, IAROW, IACOL
      REAL                 EPS, ERROR
*     ..
*     .. Array Arguments ..
      COMPLEX              A( LDA, * ), B( LDB, * )
*     ..
*
*  Purpose
*  =======
*
*  PBCMATCHK : Parallel Complex Single precision MaTriX ReSulT CHecKer.
*  A is computed parallelly (by PB-BLAS, ScaLAPACK routines) and
*  B is computed sequentially (by BLAS, LAPACK routines).
*
*  Arguments
*  =========
*
*  ICONTXT  (input) INTEGER
*           ICONTXT is the BLACS mechanism for partitioning communication
*           space.  A defining property of a context is that a message in
*           a context cannot be sent or received in another context.  The
*           BLACS context includes the definition of a grid, and each
*           process' coordinates in it.
*
*  M        (input) INTEGER
*           Number of rows of the global matrices A and B. M  >= 0.
*
*  N        (input) INTEGER
*           Number of columns of the global matrices A and B. N >= 0.
*
*  MB       (input) INTEGER
*           The row block size of A. MB >= 1.
*
*  NB       (input) INTEGER
*           The column block size of A.  NB >= 1.
*
*  A        (input/output) COMPLEX, dimension ( LDA, Nq )
*           The pointer to the local piece of the global matrix A,
*           which was computed in parallel.
*
*  LDA      (input) INTEGER
*           Leading Dimension of A.  LDA >= MAX(1,Mp)
*
*  B        (output) COMPLEX, dimension ( LDB, N )
*           The pointer to the global matrix B, which was computed
*           sequentially.
*
*  LDB      (input) INTEGER
*           Leading Dimension of B.  LDB >= MAX(1,M).
*
*  IAROW    (input) INTEGER
*           Row processor which holds the first block of the matrix.
*           0 <= IAROW < NPROW.
*
*  IACOL    (input) INTEGER
*           Column processor which holds the first block of the matrix.
*           0 <= IACOL < NPCOL.
*
*  EPS      (input) REAL
*           Endurable (tolerable) error given by user.
*
*  ERROR    (output) REAL
*           If ERROR <= EPS, the parallelly computed results are
*           correct.  Otherwise, the results are not correct.
*
*  =====================================================================
*
*     ..
*     .. Parameters ..
      REAL               ZERO
      PARAMETER          ( ZERO = 0.0E+0 )
*     ..
*     .. Local Scalars ..
      INTEGER            NPROW, NPCOL, MYROW, MYCOL, MRROW, MRCOL, INFO
      INTEGER            MP, NQ, I1, I2, J1, J2, IP, IS, JP, JS, IX
*     ..
*     .. External Functions ..
      INTEGER            ICEIL, NUMROC, ICAMAX
      EXTERNAL           ICEIL, NUMROC, ICAMAX
*     ..
*     .. External Subroutines ..
      EXTERNAL           PXERBLA, SGAMX2D, BLACS_GRIDINFO
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MOD, ABS
*
      CALL BLACS_GRIDINFO( ICONTXT, NPROW, NPCOL, MYROW, MYCOL )
*
*     .. Check input parameters ..
*
      INFO = 0
      IF(      M    .LT.0                     ) THEN
         INFO = 2
      ELSE IF( N    .LT.0                     ) THEN
         INFO = 3
      ELSE IF( MB   .LT.1                     ) THEN
         INFO = 4
      ELSE IF( NB   .LT.1                     ) THEN
         INFO = 5
      ELSE IF( LDB  .LT.M                     ) THEN
         INFO = 9
      ELSE IF( IAROW.LT.0 .OR. IAROW.GE.NPROW ) THEN
         INFO = 10
      ELSE IF( IACOL.LT.0 .OR. IACOL.GE.NPCOL ) THEN
         INFO = 11
      END IF
*
   10 CONTINUE
      IF( INFO.NE.0 ) THEN
         CALL PXERBLA( ICONTXT, 'PCMATCHK ', INFO )
         RETURN
      END IF
*     ..
*     .. Executable Statements ..
*
      MRROW = MOD( NPROW+MYROW-IAROW, NPROW )
      MRCOL = MOD( NPCOL+MYCOL-IACOL, NPCOL )
      MP    = NUMROC( M, MB, MRROW, 0, NPROW )
      NQ    = NUMROC( N, NB, MRCOL, 0, NPCOL )
      ERROR = ZERO
*
      IF( LDA.LT.MP ) THEN
        INFO = 7
        GO TO 10
      END IF
*
      IF( MP.EQ.0 .OR. NQ.EQ.0 ) GO TO 70
*
      JP = 0
      JS = MRCOL * NB
*
      DO 60 J2 = 1, ICEIL( NQ, NB )
         DO 50 J1 = 1, NB
            IF( JP+J1.GT.NQ .OR. JS+J1.GT.N ) GO TO 70
            IP = 0
            IS = MRROW * MB
*
            DO 30 I2 = 1, ICEIL( MP, MB )
               DO 20 I1 = 1, MB
                  IF( IP+I1.GT.MP .OR. IS+I1.GT.M ) GO TO 40
                  A( IP+I1, JP+J1 ) = A( IP+I1, JP+J1 ) -
     $                                B( IS+I1, JS+J1 )
   20          CONTINUE
               IP = IP + MB
               IS = IS + NPROW*MB
   30       CONTINUE
*
   40       CONTINUE
            IX = ICAMAX( MP, A( 1, JP+J1 ), 1 )
            ERROR = MAX( ERROR, ABS( A( IX, JP+J1 ) ) )
            IF( ERROR.GT.EPS ) GO TO 70
   50    CONTINUE
*
         JP = JP + NB
         JS = JS + NPCOL*NB
   60 CONTINUE
*
   70 CONTINUE
      CALL SGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, ERROR, 1, I1, I2, 1,
     $              -1, -1 )
*
      RETURN
*
*     END of PCMATCHK
*
      END
