      SUBROUTINE PCUNMBR( VECT, SIDE, TRANS, M, N, K, A, IA, JA, DESCA,
     $                    TAU, C, IC, JC, DESCC, WORK, LWORK, INFO )
*
*  -- ScaLAPACK routine (version 1.0) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     February 28, 1995
*
*     .. Scalar Arguments ..
      CHARACTER          SIDE, TRANS, VECT
      INTEGER            IA, IC, INFO, JA, JC, K, LWORK, M, N
*     ..
*     .. Array Arguments ..
      INTEGER            DESCA( * ), DESCC( * )
      COMPLEX            A( * ), C( * ), TAU( * ), WORK( * )
*     ..
*
*  Purpose
*  =======
*
*  If VECT = 'Q', PCUNMBR overwrites the general complex distributed
*  M-by-N matrix sub( C ) = C(IC:IC+M-1,JC:JC+N-1) with
*
*                       SIDE = 'L'           SIDE = 'R'
*  TRANS = 'N':      Q * sub( C )          sub( C ) * Q
*  TRANS = 'C':      Q**H * sub( C )       sub( C ) * Q**H
*
*  If VECT = 'P', PCUNMBR overwrites sub( C ) with
*
*                       SIDE = 'L'           SIDE = 'R'
*  TRANS = 'N':      P * sub( C )          sub( C ) * P
*  TRANS = 'C':      P**H * sub( C )       sub( C ) * P**H
*
*  Here Q and P**H are the unitary distributed matrices determined by
*  PCGEBRD when reducing a complex distributed matrix A(IA:*,JA:*) to
*  bidiagonal form: A(IA:*,JA:*) = Q * B * P**H. Q and P**H are defined
*  as products of elementary reflectors H(i) and G(i) respectively.
*
*  Let nq = m if SIDE = 'L' and nq = n if SIDE = 'R'. Thus nq is the
*  order of the unitary matrix Q or P**H that is applied.
*
*  If VECT = 'Q', A(IA:*,JA:*) is assumed to have been an NQ-by-K
*  matrix:
*  if nq >= k, Q = H(1) H(2) . . . H(k);
*  if nq < k, Q = H(1) H(2) . . . H(nq-1).
*
*  If VECT = 'P', A(IA:*,JA:*) is assumed to have been a K-by-NQ
*  matrix:
*  if k < nq, P = G(1) G(2) . . . G(k);
*  if k >= nq, P = G(1) G(2) . . . G(nq-1).
*
*  Notes
*  =====
*
*  A description vector is associated with each 2D block-cyclicly dis-
*  tributed matrix.  This vector stores the information required to
*  establish the mapping between a matrix entry and its corresponding
*  process and memory location.
*
*  In the following comments, the character _ should be read as
*  "of the distributed matrix".  Let A be a generic term for any 2D
*  block cyclicly distributed matrix.  Its description vector is DESCA:
*
*  NOTATION        STORED IN  EXPLANATION
*  --------------- ---------- ------------------------------------------
*  M_A    (global) DESCA( 1 ) The number of rows in the distributed
*                             matrix.
*  N_A    (global) DESCA( 2 ) The number of columns in the distributed
*                             matrix.
*  MB_A   (global) DESCA( 3 ) The blocking factor used to distribute
*                             the rows of the matrix.
*  NB_A   (global) DESCA( 4 ) The blocking factor used to distribute
*                             the columns of the matrix.
*  RSRC_A (global) DESCA( 5 ) The process row over which the first row
*                             of the matrix is distributed.
*  CSRC_A (global) DESCA( 6 ) The process column over which the first
*                             column of the matrix is distributed.
*  CTXT_A (global) DESCA( 7 ) The BLACS context handle, indicating the
*                             BLACS process grid A is distributed over.
*                             The context itself is global, but the handle
*                             (the integer value) may vary.
*  LLD_A  (local)  DESCA( 8 ) The leading dimension of the local array
*                             storing the local blocks of the distri-
*                             buted matrix A. LLD_A >= MAX(1,LOCp(M_A)).
*
*  Let K be the number of rows or columns of a distributed matrix,
*  and assume that its process grid has dimension p x q.
*  LOCp( K ) denotes the number of elements of K that a process
*  would receive if K were distributed over the p processes of its
*  process column.
*  Similarly, LOCq( K ) denotes the number of elements of K that a
*  process would receive if K were distributed over the q processes of
*  its process row.
*  The values of LOCp() and LOCq() may be determined via a call to the
*  ScaLAPACK tool function, NUMROC:
*          LOCp( M ) = NUMROC( M, MB_A, MYROW, RSRC_A, NPROW ),
*          LOCq( N ) = NUMROC( N, NB_A, MYCOL, CSRC_A, NPCOL ).
*
*  This routine requires square block decomposition ( MB_A = NB_A ).
*
*  Arguments
*  =========
*
*  VECT    (global input) CHARACTER
*          = 'Q': apply Q or Q**H;
*          = 'P': apply P or P**H.
*
*  SIDE    (global input) CHARACTER
*          = 'L': apply Q, Q**H, P or P**H from the Left;
*          = 'R': apply Q, Q**H, P or P**H from the Right.
*
*  TRANS   (global input) CHARACTER
*          = 'N':  No transpose, apply Q or P;
*          = 'C':  Conjugate transpose, apply Q**H or P**H.
*
*  M       (global input) INTEGER
*          The number of rows to be operated on i.e the number of rows
*          of the distributed submatrix sub( C ). M >= 0.
*
*  N       (global input) INTEGER
*          The number of columns to be operated on i.e the number of
*          columns of the distributed submatrix sub( C ). N >= 0.
*
*  K       (global input) INTEGER
*          If VECT = 'Q', the number of columns in the original
*          distributed matrix reduced by PCGEBRD.
*          If VECT = 'P', the number of rows in the original
*          distributed matrix reduced by PCGEBRD.
*          K >= 0.
*
*  A       (local input) COMPLEX pointer into the local memory
*          to an array of dimension (LLD_A,LOCq(JA+MIN(NQ,K)-1)) if
*          VECT='Q', and (LLD_A,LOCq(JA+NQ-1)) if VECT = 'P'. NQ = M
*          if SIDE = 'L', and NQ = N otherwise. The vectors which
*          define the elementary reflectors H(i) and G(i), whose
*          products determine the matrices Q and P, as returned by
*          PCGEBRD.
*          If VECT = 'Q', LLD_A >= max(1,LOCp(IA+NQ-1));
*          if VECT = 'P', LLD_A >= max(1,LOCp(IA+MIN(NQ,K)-1)).
*
*  IA      (global input) INTEGER
*          A's global row index, which points to the beginning of the
*          submatrix which is to be operated on.
*
*  JA      (global input) INTEGER
*          A's global column index, which points to the beginning of
*          the submatrix which is to be operated on.
*
*  DESCA   (global and local input) INTEGER array of dimension 8
*          The array descriptor for the distributed matrix A.
*
*  TAU     (local input) COMPLEX array, dimension
*          LOCq(JA+MIN(NQ,K)-1) if VECT = 'Q', LOCp(IA+MIN(NQ,K)-1) if
*          VECT = 'P', TAU(i) must contain the scalar factor of the
*          elementary  reflector H(i) or G(i), which determines Q or P,
*          as returned by PDGEBRD in its array argument TAUQ or TAUP.
*          TAU is tied to the distributed matrix A.
*
*  C       (local input/local output) COMPLEX pointer into the
*          local memory to an array of dimension (LLD_C,LOCq(JC+N-1)).
*          On entry, the local pieces of the distributed matrix sub( C ).
*          On exit, sub( C ) is overwritten by Q*sub( C ) or Q'*sub( C )
*          or sub( C )*Q' or sub( C )*Q.
*
*  IC      (global input) INTEGER
*          C's global row index, which points to the beginning of the
*          submatrix which is to be operated on.
*
*  JC      (global input) INTEGER
*          C's global column index, which points to the beginning of
*          the submatrix which is to be operated on.
*
*  DESCC   (global and local input) INTEGER array of dimension 8
*          The array descriptor for the distributed matrix C.
*
*  WORK    (local workspace) COMPLEX array, dimension (LWORK)
*          On exit, WORK(1) returns the minimal and optimal LWORK.
*
*  LWORK   (local input) INTEGER
*          The dimension of the array WORK.
*          If SIDE = 'L',
*            NQ = M;
*            if( (VECT = 'Q' and NQ >= K) or (VECT <> 'Q' and NQ > K) ),
*               IAA=IA; JAA=JA; MI=M; NI=N; ICC=IC; JCC=JC;
*            else
*               IAA=IA+1; JAA=JA; MI=M-1; NI=N; ICC=IC+1; JCC=JC;
*            end if
*          else if SIDE = 'R',
*            NQ = N;
*            if( (VECT = 'Q' and NQ >= K) or (VECT <> 'Q' and NQ > K) ),
*               IAA=IA; JAA=JA; MI=M; NI=N; ICC=IC; JCC=JC;
*            else
*               IAA=IA; JAA=JA+1; MI=M; NI=N-1; ICC=IC; JCC=JC+1;
*            end if
*          end if
*
*          If VECT = 'Q',
*            If SIDE = 'L',
*              LWORK >= MAX( (NB_A*(NB_A-1))/2, (NqC0 + MpC0)*NB_A ) +
*                       NB_A * NB_A
*            else if SIDE = 'R',
*              LWORK >= MAX( (NB_A*(NB_A-1))/2, ( NqC0 + MAX( NpA0 +
*                       NUMROC( NUMROC( NI+ICOFFC, NB_A, 0, 0, NPCOL ),
*                               NB_A, 0, 0, LCMQ ), MpC0 ) )*NB_A ) +
*                       NB_A * NB_A
*            end if
*          else if VECT <> 'Q',
*            if SIDE = 'L',
*              LWORK >= MAX( (MB_A*(MB_A-1))/2, ( MpC0 + MAX( MqA0 +
*                       NUMROC( NUMROC( MI+IROFFC, MB_A, 0, 0, NPROW ),
*                               MB_A, 0, 0, LCMP ), NqC0 ) )*MB_A ) +
*                       MB_A * MB_A
*            else if SIDE = 'R',
*              LWORK >= MAX( (MB_A*(MB_A-1))/2, (MpC0 + NqC0)*MB_A ) +
*                       MB_A * MB_A
*            end if
*          end if
*
*          where LCMP = LCM / NPROW, LCMQ = LCM / NPCOL, with
*          LCM = ICLM( NPROW, NPCOL ),
*
*          IROFFA = MOD( IAA-1, MB_A ), ICOFFA = MOD( JAA-1, NB_A ),
*          IAROW = INDXG2P( IAA, MB_A, MYROW, RSRC_A, NPROW ),
*          IACOL = INDXG2P( JAA, NB_A, MYCOL, CSRC_A, NPCOL ),
*          MqA0 = NUMROC( MI+IROFFA, NB_A, MYCOL, IACOL, NPCOL ),
*          NpA0 = NUMROC( NI+IROFFA, MB_A, MYROW, IAROW, NPROW ),
*
*          IROFFC = MOD( ICC-1, MB_C ), ICOFFC = MOD( JCC-1, NB_C ),
*          ICROW = INDXG2P( ICC, MB_C, MYROW, RSRC_C, NPROW ),
*          ICCOL = INDXG2P( JCC, NB_C, MYCOL, CSRC_C, NPCOL ),
*          MpC0 = NUMROC( MI+IROFFC, MB_C, MYROW, ICROW, NPROW ),
*          NqC0 = NUMROC( NI+ICOFFC, NB_C, MYCOL, ICCOL, NPCOL ),
*
*          INDXG2P and NUMROC are ScaLAPACK tool functions;
*          MYROW, MYCOL, NPROW and NPCOL can be determined by calling
*          the subroutine BLACS_GRIDINFO.
*
*  INFO    (global output) INTEGER
*          = 0:  successful exit
*          < 0:  If the i-th argument is an array and the j-entry had
*                an illegal value, then INFO = -(i*100+j), if the i-th
*                argument is a scalar and had an illegal value, then
*                INFO = -i.
*
*  =====================================================================
*
*  =====================================================================
*
*  Alignment requirements
*  ======================
*
*  The distributed submatrices A(IA:*, JA:*) and C(IC:IC+M-1,JC:JC+N-1)
*  must verify some alignment properties, namely the following
*  expressions should be true:
*
*  If VECT = 'Q',
*    If SIDE = 'L',
*      ( MB_A.EQ.MB_C .AND. IROFFA.EQ.IROFFC .AND. IAROW.EQ.ICROW )
*     If SIDE = 'R',
*      ( MB_A.EQ.NB_A .AND. MB_A.EQ.NB_C .AND. IROFFA.EQ.ICOFFC )
*  else
*     If SIDE = 'L',
*       ( MB_A.EQ.NB_A .AND. MB_A.EQ.MB_C .AND. ICOFFA.EQ.IROFFC )
*     If SIDE = 'R',
*       ( NB_A.EQ.NB_C .AND. ICOFFA.EQ.ICOFFC .AND. IACOL.EQ.ICCOL )
*  end if
*
*  =====================================================================
*
*     .. Local Scalars ..
      LOGICAL            APPLYQ, LEFT, NOTRAN
      CHARACTER          TRANST
      INTEGER            IAA, IACOL, IAROW, ICC, ICCOL, ICOFFA, ICOFFC,
     $                   ICROW, ICTXT, IINFO, IROFFA, IROFFC, JAA, JCC,
     $                   LCM, LCMP, LCMQ, LWMIN, MI, MPC0, MQA0, MYCOL,
     $                   MYROW, NI, NPA0, NPCOL, NPROW, NQ, NQC0
*     ..
*     .. Local Arrays ..
      INTEGER            IDUM1( 4 ), IDUM2( 4 )
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GRIDINFO, CHK1MAT, PCHK2MAT, PCUNMLQ,
     $                   PCUNMQR, PXERBLA
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
      INTEGER            ICEIL, ILCM, INDXG2P, NUMROC
      EXTERNAL           ICEIL, ILCM, INDXG2P, LSAME, NUMROC
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ICHAR, MAX, MOD, REAL
*     ..
*     .. Executable Statements ..
*
*     Get grid parameters
*
      ICTXT = DESCA( 7 )
      CALL BLACS_GRIDINFO( ICTXT, NPROW, NPCOL, MYROW, MYCOL )
*
*     Test the input parameters
*
      INFO = 0
      IF( NPROW.EQ.-1 ) THEN
         INFO = -1007
      ELSE
         APPLYQ = LSAME( VECT, 'Q' )
         LEFT = LSAME( SIDE, 'L' )
         NOTRAN = LSAME( TRANS, 'N' )
*
*        NQ is the order of Q or P
*
         IF( LEFT ) THEN
            NQ = M
            IF( ( APPLYQ .AND. NQ.GE.K ) .OR.
     $          ( .NOT.APPLYQ .AND. NQ.GT.K ) ) THEN
               IAA = IA
               JAA = JA
               MI = M
               NI = N
               ICC = IC
               JCC = JC
            ELSE
               IAA = IA + 1
               JAA = JA
               MI = M - 1
               NI = N
               ICC = IC + 1
               JCC = JC
            END IF
*
            IF( APPLYQ ) THEN
               CALL CHK1MAT( M, 4, K, 6, IA, JA, DESCA, 10, INFO )
            ELSE
               CALL CHK1MAT( K, 6, M, 4, IA, JA, DESCA, 10, INFO )
            END IF
         ELSE
            NQ = N
            IF( ( APPLYQ .AND. NQ.GE.K ) .OR.
     $          ( .NOT.APPLYQ .AND. NQ.GT.K ) ) THEN
               IAA = IA
               JAA = JA
               MI = M
               NI = N
               ICC = IC
               JCC = JC
            ELSE
               IAA = IA
               JAA = JA + 1
               MI = M
               NI = N - 1
               ICC = IC
               JCC = JC + 1
            END IF
*
            IF( APPLYQ ) THEN
               CALL CHK1MAT( N, 5, K, 6, IA, JA, DESCA, 10, INFO )
            ELSE
               CALL CHK1MAT( K, 6, N, 5, IA, JA, DESCA, 10, INFO )
            END IF
         END IF
         CALL CHK1MAT( M, 4, N, 5, IC, JC, DESCC, 15, INFO )
*
         IF( INFO.EQ.0 ) THEN
            IROFFA = MOD( IAA-1, DESCA( 3 ) )
            ICOFFA = MOD( JAA-1, DESCA( 4 ) )
            IROFFC = MOD( ICC-1, DESCC( 3 ) )
            ICOFFC = MOD( JCC-1, DESCC( 4 ) )
            IACOL = INDXG2P( JAA, DESCA( 4 ), MYCOL, DESCA( 6 ), NPCOL )
            IAROW = INDXG2P( IAA, DESCA( 3 ), MYROW, DESCA( 5 ), NPROW )
            ICROW = INDXG2P( ICC, DESCC( 3 ), MYROW, DESCC( 5 ), NPROW )
            ICCOL = INDXG2P( JCC, DESCC( 4 ), MYCOL, DESCC( 6 ), NPCOL )
            MPC0 = NUMROC( MI+IROFFC, DESCC( 3 ), MYROW, ICROW,  NPROW )
            NQC0 = NUMROC( NI+ICOFFC, DESCC( 4 ), MYCOL, ICCOL, NPCOL )
*
            IF( APPLYQ ) THEN
               IF( LEFT ) THEN
                  LWMIN = MAX( ( DESCA( 4 ) * ( DESCA( 4 ) - 1 ) ) / 2,
     $                         ( MPC0 + NQC0 ) * DESCA( 4 ) ) +
     $                    DESCA( 4 ) * DESCA( 4 )
               ELSE
                  NPA0 = NUMROC( NI+IROFFA, DESCA( 3 ), MYROW, IAROW,
     $                           NPROW )
                  LCM = ILCM( NPROW, NPCOL )
                  LCMQ = LCM / NPCOL
                  LWMIN =  MAX( ( DESCA( 4 ) * ( DESCA( 4 ) - 1 ) ) / 2,
     $                          ( NQC0 + MAX( NPA0 + NUMROC( NUMROC(
     $                            NI+ICOFFC, DESCA( 4 ), 0, 0, NPCOL ),
     $                            DESCA( 4 ), 0, 0, LCMQ ), MPC0 ) ) *
     $                            DESCA( 4 ) ) + DESCA( 4 ) * DESCA( 4 )
               END IF
            ELSE
*
               IF( LEFT ) THEN
                  MQA0 = NUMROC( MI+ICOFFA, DESCA( 4 ), MYCOL, IACOL,
     $                           NPCOL )
                  LCM = ILCM( NPROW, NPCOL )
                  LCMP = LCM / NPROW
                  LWMIN =  MAX( ( DESCA( 3 ) * ( DESCA( 3 ) - 1 ) ) / 2,
     $                          ( MPC0 + MAX( MQA0 + NUMROC( NUMROC(
     $                            MI+IROFFC, DESCA( 3 ), 0, 0, NPROW ),
     $                            DESCA( 3 ), 0, 0, LCMP ), NQC0 ) ) *
     $                            DESCA( 3 ) ) + DESCA( 3 ) * DESCA( 3 )
               ELSE
                  LWMIN = MAX( ( DESCA( 3 ) * ( DESCA( 3 ) - 1 ) ) / 2,
     $                         ( MPC0 + NQC0 ) * DESCA( 3 ) ) +
     $                    DESCA( 3 ) * DESCA( 3 )
               END IF
*
            END IF
*
            WORK( 1 ) = REAL( LWMIN )
            IF( .NOT.APPLYQ .AND. .NOT.LSAME( VECT, 'P' ) ) THEN
               INFO = -1
            ELSE IF( .NOT.LEFT .AND. .NOT.LSAME( SIDE, 'R' ) ) THEN
               INFO = -2
            ELSE IF( .NOT.NOTRAN .AND. .NOT.LSAME( TRANS, 'C' ) ) THEN
               INFO = -3
            ELSE IF( K.LT.0 ) THEN
               INFO = -6
            ELSE IF( DESCA( 3 ).NE.DESCA( 4 ) ) THEN
               INFO = -1004
            ELSE IF( APPLYQ .AND. LEFT .AND. IROFFA.NE.IROFFC ) THEN
               INFO = -13
            ELSE IF( .NOT.APPLYQ .AND. LEFT .AND.
     $               ICOFFA.NE.IROFFC ) THEN
               INFO = -13
            ELSE IF( APPLYQ .AND. LEFT .AND. IAROW.NE.ICROW ) THEN
               INFO = -13
            ELSE IF( .NOT.APPLYQ .AND. .NOT.LEFT .AND.
     $               IACOL.NE.ICCOL ) THEN
               INFO = -14
            ELSE IF( APPLYQ .AND. .NOT.LEFT .AND.
     $               IROFFA.NE.ICOFFC ) THEN
               INFO = -14
            ELSE IF( .NOT.APPLYQ .AND. .NOT.LEFT .AND.
     $               ICOFFA.NE.ICOFFC ) THEN
               INFO = -14
            ELSE IF( APPLYQ .AND. LEFT .AND.
     $               DESCA( 3 ).NE.DESCC( 3 ) ) THEN
               INFO = -1503
            ELSE IF( .NOT.APPLYQ .AND. LEFT .AND.
     $               DESCA( 3 ).NE.DESCC( 3 ) ) THEN
               INFO = -1503
            ELSE IF( APPLYQ .AND. .NOT.LEFT .AND.
     $               DESCA( 4 ).NE.DESCC( 4 ) ) THEN
               INFO = -1504
            ELSE IF( .NOT.APPLYQ .AND. .NOT.LEFT .AND.
     $               DESCA( 4 ).NE.DESCC( 4 ) ) THEN
               INFO = -1504
            ELSE IF( LWORK.LT.LWMIN ) THEN
               INFO = -17
            END IF
         END IF
*
         IF( APPLYQ ) THEN
            IDUM1( 1 ) = ICHAR( 'Q' )
         ELSE
            IDUM1( 1 ) = ICHAR( 'P' )
         END IF
         IDUM2( 1 ) = 1
         IF( LEFT ) THEN
            IDUM1( 2 ) = ICHAR( 'L' )
         ELSE
            IDUM1( 2 ) = ICHAR( 'R' )
         END IF
         IDUM2( 2 ) = 2
         IF( NOTRAN ) THEN
            IDUM1( 3 ) = ICHAR( 'N' )
         ELSE
            IDUM1( 3 ) = ICHAR( 'C' )
         END IF
         IDUM2( 3 ) = 3
         IDUM1( 4 ) = K
         IDUM2( 4 ) = 6
         IF( APPLYQ ) THEN
            IF( LEFT ) THEN
               CALL PCHK2MAT( M, 4, K, 6, IA, JA, DESCA, 10, M, 4, N,
     $                        5, IC, JC, DESCC, 15, 4, IDUM1, IDUM2,
     $                        INFO )
            ELSE
               CALL PCHK2MAT( N, 5, K, 6, IA, JA, DESCA, 10, M, 4, N,
     $                        5, IC, JC, DESCC, 15, 4, IDUM1, IDUM2,
     $                        INFO )
            END IF
         ELSE
            IF( LEFT ) THEN
               CALL PCHK2MAT( K, 6, M, 4, IA, JA, DESCA, 10, M, 4, N,
     $                        5, IC, JC, DESCC, 15, 4, IDUM1, IDUM2,
     $                        INFO )
            ELSE
               CALL PCHK2MAT( K, 6, N, 5, IA, JA, DESCA, 10, M, 4, N,
     $                        5, IC, JC, DESCC, 15, 4, IDUM1, IDUM2,
     $                        INFO )
            END IF
         END IF
      END IF
*
      IF( INFO.NE.0 ) THEN
         CALL PXERBLA( ICTXT, 'PCUNMBR', -INFO )
         RETURN
      END IF
*
*     Quick return if possible
*
      IF( M.EQ.0 .OR. N.EQ.0 )
     $   RETURN
*
      IF( APPLYQ ) THEN
*
*        Apply Q
*
         IF( NQ.GE.K ) THEN
*
*           Q was determined by a call to PCGEBRD with nq >= k
*
            CALL PCUNMQR( SIDE, TRANS, M, N, K, A, IA, JA, DESCA, TAU,
     $                    C, IC, JC, DESCC, WORK, LWORK, IINFO )
         ELSE IF( NQ.GT.1 ) THEN
*
*           Q was determined by a call to PCGEBRD with nq < k
*
            CALL PCUNMQR( SIDE, TRANS, MI, NI, NQ-1, A, IA+1, JA, DESCA,
     $                    TAU, C, ICC, JCC, DESCC, WORK, LWORK, IINFO )
         END IF
      ELSE
*
*        Apply P
*
         IF( NOTRAN ) THEN
            TRANST = 'C'
         ELSE
            TRANST = 'N'
         END IF
         IF( NQ.GT.K ) THEN
*
*           P was determined by a call to PCGEBRD with nq > k
*
            CALL PCUNMLQ( SIDE, TRANST, M, N, K, A, IA, JA, DESCA, TAU,
     $                    C, IC, JC, DESCC, WORK, LWORK, IINFO )
         ELSE IF( NQ.GT.1 ) THEN
*
*           P was determined by a call to PCGEBRD with nq <= k
*
            CALL PCUNMLQ( SIDE, TRANST, MI, NI, NQ-1, A, IA, JA+1,
     $                    DESCA, TAU, C, ICC, JCC, DESCC, WORK, LWORK,
     $                    IINFO )
         END IF
      END IF
*
      WORK( 1 ) = REAL( LWMIN )
*
      RETURN
*
*     End of PCUNMBR
*
      END
