      SUBROUTINE PDLARED1D( N, IA, JA, DESC, BYCOL, BYALL, WORK, LWORK )
*
*
*  -- ScaLAPACK routine (version 1.0) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     February 28, 1995
*
*     .. Scalar Arguments ..
      INTEGER            IA, JA, LWORK, N
*     ..
*     .. Array Arguments ..
      INTEGER            DESC( 8 )
      DOUBLE PRECISION   BYALL( * ), BYCOL( * ), WORK( LWORK )
*     ..
*
*  Purpose
*  =======
*
*  PDLARED1D redistributes a 1D array
*
*  It assumes that the input array, BYCOL, is distributed across
*  rows and that all process column contain the same copy of
*  BYCOL.  The output array, BYALL, will be identical on all processes
*  and will contain the entire array.
*
*  Notes
*  =====
*
*
*  A description vector is associated with each 2D block-cyclicly dis-
*  tributed matrix.  This vector stores the information required to
*  establish the mapping between a matrix entry and its corresponding
*  process and memory location.
*
*  In the following comments, the character _ should be read as
*  "of the distributed matrix".  Let A be a generic term for any 2D
*  block cyclicly distributed matrix.  Its description vector is DESCA:
*
*  NOTATION        STORED IN  EXPLANATION
*  --------------- ---------- ------------------------------------------
*  M_A    (global) DESCA( 1 ) The number of rows in the distributed
*                             matrix.
*  N_A    (global) DESCA( 2 ) The number of columns in the distributed
*                             matrix.
*  MB_A   (global) DESCA( 3 ) The blocking factor used to distribute
*                             the rows of the matrix.
*  NB_A   (global) DESCA( 4 ) The blocking factor used to distribute
*                             the columns of the matrix.
*  RSRC_A (global) DESCA( 5 ) The process row over which the first row
*                             of the matrix is distributed.
*  CSRC_A (global) DESCA( 6 ) The process column over which the first
*                             column of the matrix is distributed.
*  CTXT_A (global) DESCA( 7 ) The BLACS context handle, indicating the
*                             global context of the operation on the
*                             matrix. The context itself is global.
*  LLD_A  (local)  DESCA( 8 ) The leading dimension of the local array
*                             storing the local blocks of the distri-
*                             buted matrix A. LLD_A >= MAX(1,LOCp(M_A)).
*
*  Let K be the number of rows or columns of a distributed matrix,
*  and assume that its process grid has dimension p x q.
*  LOCp( K ) denotes the number of elements of K that a process
*  would receive if K were distributed over the p processes of its
*  process column.
*  Similarly, LOCq( K ) denotes the number of elements of K that a
*  process would receive if K were distributed over the q processes of
*  its process row.
*  The values of LOCp() and LOCq() may be determined via a call to the
*  ScaLAPACK tool function, NUMROC:
*          LOCp( M ) = NUMROC( M, MB_A, MYROW, RSRC_A, NPROW ),
*          LOCq( N ) = NUMROC( N, NB_A, MYCOL, CSRC_A, NPCOL ).
*
*  Arguments
*  =========
*
*    NP = Number of local rows in BYCOL()
*
*  N       (global input) INTEGER
*          The number of elements to be redistributed.  N >= 0.
*
*  IA      (global input) INTEGER
*          IA must be equal to 1
*
*  JA      (global input) INTEGER
*          JA must be equal to 1
*
*  DESC    (global/local input) INTEGER Array of dimension 8
*          A 2D array descirptor, which describes BYCOL
*
*  BYCOL   (local input) distributed block cyclic DOUBLE PRECISION array
*          global dimension (N), local dimension NP
*          BYCOL is distributed across the process rows
*          All process columns are assumed to contain the same value
*
*  BYALL   (global output) DOUBLE PRECISION global dimension( N )
*          local dimension (N)
*          BYALL is exactly duplicated on all processes
*          It contains the same values as BYCOL, but it is replicated
*          across all processes rather than being distributed
*
*          BYALL(i) = BYCOL( NUMROC(i,NB,MYROW,0,NPROW ) on the procs
*          whose MYROW == mod((i-1)/NB,NPROW)
*
*  WORK    (local workspace) DOUBLE PRECISION dimension (LWORK)
*          Used to hold the buffers sent from one process to another
*
*  LWORK   (local input) INTEGER size of WORK array LWORK >=
*          LWORK >= NUMROC(N, DESC(4), 0, 0, NPCOL)
*
*
*     .. Local Scalars ..
      INTEGER            ALLI, BUFLEN, I, II, MYCOL, MYROW, NB, NPCOL,
     $                   NPROW, PCOL
*     ..
*     .. External Functions ..
*
      INTEGER            NUMROC
      EXTERNAL           NUMROC
*     ..
*     .. External Subroutines ..
*
      EXTERNAL           BLACS_GRIDINFO, DCOPY, DGEBR2D, DGEBS2D
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MIN
*     ..
*     .. Executable Statements ..
*
      CALL BLACS_GRIDINFO( DESC( 7 ), NPROW, NPCOL, MYROW, MYCOL )
      NB = DESC( 3 )
*
*
      DO 30 PCOL = 0, NPCOL - 1
         BUFLEN = NUMROC( N, NB, PCOL, 0, NPCOL )
         IF( MYCOL.EQ.PCOL ) THEN
            CALL DCOPY( BUFLEN, BYCOL, 1, WORK, 1 )
            CALL DGEBS2D( DESC( 7 ), 'R', ' ', 1, BUFLEN, WORK, 1 )
         ELSE
            CALL DGEBR2D( DESC( 7 ), 'R', ' ', 1, BUFLEN, WORK, 1,
     $                    MYROW, PCOL )
         END IF
*
         ALLI = PCOL*NB
         DO 20 II = 1, BUFLEN, NB
            DO 10 I = 1, MIN( NB, BUFLEN-II+1 )
               BYALL( ALLI+I ) = WORK( II-1+I )
   10       CONTINUE
            ALLI = ALLI + NB*NPCOL
   20    CONTINUE
   30 CONTINUE
*
      RETURN
      END
