      SUBROUTINE PZLABRD( M, N, NB, A, IA, JA, DESCA, D, E, TAUQ, TAUP,
     $                    X, IX, JX, DESCX, Y, IY, JY, DESCY, WORK )
*
*  -- ScaLAPACK auxilliary routine (version 1.0) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     February 28, 1995
*
*     .. Scalar Arguments ..
      INTEGER             IA, IX, IY, JA, JX, JY, M, N, NB
*     ..
*     .. Array Arguments ..
      INTEGER             DESCA( * ), DESCX( * ), DESCY( * )
      COMPLEX*16          A( * ), D( * ), E( * ), TAUP( * ),
     $                    TAUQ( * ), X( * ), Y( * ), WORK( * )
*     ..
*
*  Purpose
*  =======
*
*  PZLABRD reduces the first NB rows and columns of a complex general
*  M-by-N distributed matrix sub( A ) = A(IA:IA+M-1,JA:JA+N-1) to upper
*  or lower bidiagonal form by an unitary transformation Q' * A * P, and
*  returns the matrices X and Y which are needed to apply the transforma-
*  tion to the unreduced part of sub( A ).
*
*  If M >= N, sub( A ) is reduced to upper bidiagonal form; if M < N, to
*  lower bidiagonal form.
*
*  This is an auxiliary routine called by PZGEBRD.
*
*  Notes
*  =====
*
*  A description vector is associated with each 2D block-cyclicly dis-
*  tributed matrix.  This vector stores the information required to
*  establish the mapping between a matrix entry and its corresponding
*  process and memory location.
*
*  In the following comments, the character _ should be read as
*  "of the distributed matrix".  Let A be a generic term for any 2D
*  block cyclicly distributed matrix.  Its description vector is DESCA:
*
*  NOTATION        STORED IN  EXPLANATION
*  --------------- ---------- ------------------------------------------
*  M_A    (global) DESCA( 1 ) The number of rows in the distributed
*                             matrix.
*  N_A    (global) DESCA( 2 ) The number of columns in the distributed
*                             matrix.
*  MB_A   (global) DESCA( 3 ) The blocking factor used to distribute
*                             the rows of the matrix.
*  NB_A   (global) DESCA( 4 ) The blocking factor used to distribute
*                             the columns of the matrix.
*  RSRC_A (global) DESCA( 5 ) The process row over which the first row
*                             of the matrix is distributed.
*  CSRC_A (global) DESCA( 6 ) The process column over which the first
*                             column of the matrix is distributed.
*  CTXT_A (global) DESCA( 7 ) The BLACS context handle, indicating the
*                             BLACS process grid A is distributed over.
*                             The context itself is global, but the handle
*                             (the integer value) may vary.
*  LLD_A  (local)  DESCA( 8 ) The leading dimension of the local array
*                             storing the local blocks of the distri-
*                             buted matrix A. LLD_A >= MAX(1,LOCp(M_A)).
*
*  Let K be the number of rows or columns of a distributed matrix,
*  and assume that its process grid has dimension p x q.
*  LOCp( K ) denotes the number of elements of K that a process
*  would receive if K were distributed over the p processes of its
*  process column.
*  Similarly, LOCq( K ) denotes the number of elements of K that a
*  process would receive if K were distributed over the q processes of
*  its process row.
*  The values of LOCp() and LOCq() may be determined via a call to the
*  ScaLAPACK tool function, NUMROC:
*          LOCp( M ) = NUMROC( M, MB_A, MYROW, RSRC_A, NPROW ),
*          LOCq( N ) = NUMROC( N, NB_A, MYCOL, CSRC_A, NPCOL ).
*
*  Arguments
*  =========
*
*  M       (global input) INTEGER
*          The number of rows to be operated on, i.e. the number of rows
*          of the distributed submatrix sub( A ). M >= 0.
*
*  N       (global input) INTEGER
*          The number of columns to be operated on, i.e. the number of
*          columns of the distributed submatrix sub( A ). N >= 0.
*
*  NB      (global input) INTEGER
*          The number of leading rows and columns of sub( A ) to be
*          reduced.
*
*  A       (local input/local output) COMPLEX*16 pointer into the
*          local memory to an array of dimension (LLD_A,LOCq(JA+N-1)).
*          On entry, this array contains the local pieces of the
*          general distributed matrix sub( A ) to be reduced. On exit,
*          the first NB rows and columns of the matrix are overwritten;
*          the rest of the distributed matrix sub( A ) is unchanged.
*          If m >= n, elements on and below the diagonal in the first NB
*            columns, with the array TAUQ, represent the unitary
*            matrix Q as a product of elementary reflectors; and
*            elements above the diagonal in the first NB rows, with the
*            array TAUP, represent the unitary matrix P as a product
*            of elementary reflectors.
*          If m < n, elements below the diagonal in the first NB
*            columns, with the array TAUQ, represent the unitary
*            matrix Q as a product of elementary reflectors, and
*            elements on and above the diagonal in the first NB rows,
*            with the array TAUP, represent the unitary matrix P as
*            a product of elementary reflectors.
*          See Further Details.
*
*  IA      (global input) INTEGER
*          A's global row index, which points to the beginning of the
*          submatrix which is to be operated on.
*
*  JA      (global input) INTEGER
*          A's global column index, which points to the beginning of
*          the submatrix which is to be operated on.
*
*  DESCA   (global and local input) INTEGER array of dimension 8
*          The array descriptor for the distributed matrix A.
*
*  D       (local output) COMPLEX*16 array, dimension
*          LOCp(IA+MIN(M,N)-1) if M >= N; LOCq(JA+MIN(M,N)-1) otherwise.
*          The distributed diagonal elements of the bidiagonal matrix
*          B: D(i) = A(ia+i-1,ja+i-1). D is tied to the distributed
*          matrix A.
*
*  E       (local output) COMPLEX*16 array, dimension
*          LOCp(IA+MIN(M,N)-1) if M >= N; LOCq(JA+MIN(M,N)-2) otherwise.
*          The distributed off-diagonal elements of the bidiagonal
*          distributed matrix B:
*          if m >= n, E(i) = A(ia+i-1,ja+i) for i = 1,2,...,n-1;
*          if m < n, E(i) = A(ia+i,ja+i-1) for i = 1,2,...,m-1.
*          E is tied to the distributed matrix A.
*
*  TAUQ    (local output) COMPLEX*16 array dimension
*          LOCq(JA+MIN(M,N)-1). The scalar factors of the elementary
*          reflectors which represent the unitary matrix Q. TAUQ is
*          tied to the distributed matrix A. See Further Details.
*
*  TAUP    (local output) COMPLEX*16 array, dimension
*          LOCp(IA+MIN(M,N)-1). The scalar factors of the elementary
*          reflectors which represent the unitary matrix P. TAUP is
*          tied to the distributed matrix A. See Further Details.
*
*  X       (local output) COMPLEX*16 pointer into the local memory
*          to an array of dimension (LLD_X,NB). On exit, the local
*          pieces of the distributed M-by-NB matrix
*          X(IX:IX+M-1,JX:JX+NB-1) required to update the unreduced
*          part of sub( A ).
*
*  IX      (global input) INTEGER
*          X's global row index, which points to the beginning of the
*          submatrix which is to be operated on.
*
*  JX      (global input) INTEGER
*          X's global column index, which points to the beginning of
*          the submatrix which is to be operated on.
*
*  DESCX   (global and local input) INTEGER array of dimension 8
*          The array descriptor for the distributed matrix X.
*
*  Y       (local output) COMPLEX*16 pointer into the local memory
*          to an array of dimension (LLD_X,NB).  On exit, the local
*          pieces of the distributed N-by-NB matrix
*          Y(IY:IY+N-1,JY:JY+NB-1) required to update the unreduced
*          part of sub( A ).
*
*  IY      (global input) INTEGER
*          Y's global row index, which points to the beginning of the
*          submatrix which is to be operated on.
*
*  JY      (global input) INTEGER
*          Y's global column index, which points to the beginning of
*          the submatrix which is to be operated on.
*
*  DESCY   (global and local input) INTEGER array of dimension 8
*          The array descriptor for the distributed matrix Y.
*
*  WORK    (local workspace) COMPLEX*16 array, dimension (LWORK)
*          LWORK >= NB_A + NQ, with
*
*          NQ = NUMROC( N+MOD( IA-1, NB_Y ), NB_Y, MYCOL, IACOL, NPCOL )
*          IACOL = INDXG2P( JA, NB_A, MYCOL, CSRC_A, NPCOL )
*
*          INDXG2P and NUMROC are ScaLAPACK tool functions;
*          MYROW, MYCOL, NPROW and NPCOL can be determined by calling
*          the subroutine BLACS_GRIDINFO.
*
*  Further Details
*  ===============
*
*  The matrices Q and P are represented as products of elementary
*  reflectors:
*
*     Q = H(1) H(2) . . . H(nb)  and  P = G(1) G(2) . . . G(nb)
*
*  Each H(i) and G(i) has the form:
*
*     H(i) = I - tauq * v * v'  and G(i) = I - taup * u * u'
*
*  where tauq and taup are complex scalars, and v and u are complex
*  vectors.
*
*  If m >= n, v(1:i-1) = 0, v(i) = 1, and v(i:m) is stored on exit in
*  A(ia+i-1:ia+m-1,ja+i-1); u(1:i) = 0, u(i+1) = 1, and u(i+1:n) is
*  stored on exit in A(ia+i-1,ja+i:ja+n-1); tauq is stored in
*  TAUQ(ja+i-1) and taup in TAUP(ia+i-1).
*
*  If m < n, v(1:i) = 0, v(i+1) = 1, and v(i+1:m) is stored on exit in
*  A(ia+i+1:ia+m-1,ja+i-1); u(1:i-1) = 0, u(i) = 1, and u(i:n) is
*  stored on exit in A(ia+i-1,ja+i:ja+n-1); tauq is stored in
*  TAUQ(ja+i-1) and taup in TAUP(ia+i-1).
*
*  The elements of the vectors v and u together form the m-by-nb matrix
*  V and the nb-by-n matrix U' which are needed, with X and Y, to apply
*  the transformation to the unreduced part of the matrix, using a block
*  update of the form:  sub( A ) := sub( A ) - V*Y' - X*U'.
*
*  The contents of sub( A ) on exit are illustrated by the following
*  examples with nb = 2:
*
*  m = 6 and n = 5 (m > n):          m = 5 and n = 6 (m < n):
*
*    (  1   1   u1  u1  u1 )           (  1   u1  u1  u1  u1  u1 )
*    (  v1  1   1   u2  u2 )           (  1   1   u2  u2  u2  u2 )
*    (  v1  v2  a   a   a  )           (  v1  1   a   a   a   a  )
*    (  v1  v2  a   a   a  )           (  v1  v2  a   a   a   a  )
*    (  v1  v2  a   a   a  )           (  v1  v2  a   a   a   a  )
*    (  v1  v2  a   a   a  )
*
*  where a denotes an element of the original matrix which is unchanged,
*  vi denotes an element of the vector defining H(i), and ui an element
*  of the vector defining G(i).
*
*  =====================================================================
*
*     .. Parameters ..
      COMPLEX*16         ONE, ZERO
      PARAMETER          ( ONE = ( 1.0D+0, 0.0D+0 ),
     $                   ZERO = ( 0.0D+0, 0.0D+0 ) )
*     ..
*     .. Local Scalars ..
      INTEGER            I, IACOL, IAROW, ICTXT, II, IJ, IPY, IW, J, JJ,
     $                   JWY, K, MP, MYCOL, MYROW, NPCOL, NPROW, NQ
      COMPLEX*16         TAU
      INTEGER            DESCW( 8 ), DESCWY( 8 )
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GRIDINFO, DESCSET, INFOG2L, PZCOPY,
     $                   PZELSET, PZGEMV, PZLACGV, PZLARFG,
     $                   PZSCAL, ZGEBR2D, ZGEBS2D
*     ..
*     .. External Functions ..
      INTEGER            NUMROC
      EXTERNAL           NUMROC
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MIN, MOD
*     ..
*     .. Executable Statements ..
*
*     Quick return if possible
*
      IF( M.LE.0 .OR. N.LE.0 )
     $   RETURN
*
      ICTXT = DESCA( 7 )
      CALL BLACS_GRIDINFO( ICTXT, NPROW, NPCOL, MYROW, MYCOL )
      CALL INFOG2L( IA, JA, DESCA, NPROW, NPCOL, MYROW, MYCOL, II, JJ,
     $              IAROW, IACOL )
      MP = MAX( 1, NUMROC( IA+M-1, DESCA( 3 ), MYROW, DESCA( 5 ),
     $          NPROW ) )
      NQ = MAX( 1, NUMROC( JA+N-1, DESCA( 4 ), MYCOL, DESCA( 6 ),
     $          NPCOL ) )
      IPY = DESCA( 3 ) + 1
      IW = MOD( IA-1, DESCA( 4 ) ) + 1
*
      CALL DESCSET( DESCWY, 1, N+MOD( IA-1, DESCY( 4 ) ), 1, DESCA( 4 ),
     $              IAROW, IACOL, ICTXT, 1 )
      CALL DESCSET( DESCW, DESCA( 3 ), 1, DESCA( 3 ), 1, IAROW, IACOL,
     $              ICTXT, DESCA( 3 ) )
*
      IF( M.GE.N ) THEN
*
*        Reduce to upper bidiagonal form
*
         DO 10 K = 1, NB
            I = IA + K - 1
            J = JA + K - 1
            JWY = IW + K
*
*           Update A(i:ia+m-1,j)
*
            IF( K.GT.1 ) THEN
               CALL PZGEMV( 'No transpose', M-K+1, K-1, -ONE, A, I, JA,
     $                      DESCA, Y, IY, JY+K-1, DESCY, 1, ONE, A, I,
     $                      J, DESCA, 1 )
               CALL PZGEMV( 'No transpose', M-K+1, K-1, -ONE, X, IX+K-1,
     $                      JX, DESCX, A, IA, J, DESCA, 1, ONE, A, I, J,
     $                      DESCA, 1 )
               CALL PZELSET( A, I-1, J, DESCA, E( IJ ) )
            END IF
*
*           Generate reflection Q(i) to annihilate A(i+1:ia+m-1,j)
*
            IJ = MIN( JJ+K-1, NQ )
            CALL PZLARFG( M-K+1, D( IJ ), I, J, A, I+1, J, DESCA, 1,
     $                    TAUQ )
            CALL PZELSET( A, I, J, DESCA, ONE )
*
*           Compute Y(IA+I:IA+N-1,J)
*
            CALL PZGEMV( 'Conjugate transpose', M-K+1, N-K, ONE, A, I,
     $                   J+1, DESCA, A, I, J, DESCA, 1, ZERO,
     $                   WORK( IPY ), 1, JWY, DESCWY, DESCWY( 1 ) )
            CALL PZGEMV( 'Conjugate transpose', M-K+1, K-1, ONE, A, I,
     $                   JA, DESCA, A, I, J, DESCA, 1, ZERO, WORK, IW,
     $                   1, DESCW, 1 )
            CALL PZGEMV( 'Conjugate transpose', K-1, N-K, -ONE, Y, IY,
     $                   JY+K, DESCY, WORK, IW, 1, DESCW, 1, ONE,
     $                   WORK( IPY ), 1, JWY, DESCWY, DESCWY( 1 ) )
            CALL PZGEMV( 'Conjugate transpose', M-K+1, K-1, ONE, X,
     $                   IX+K-1, JX, DESCX, A, I, J, DESCA, 1, ZERO,
     $                   WORK, IW, 1, DESCW, 1 )
            CALL PZGEMV( 'Conjugate transpose', K-1, N-K, -ONE, A, IA,
     $                   J+1, DESCA, WORK, IW, 1, DESCW, 1, ONE,
     $                   WORK( IPY ), 1, JWY, DESCWY, DESCWY( 1 ) )
*
            IF( MYROW.EQ.IAROW ) THEN
               IF( MYCOL.EQ.IACOL ) THEN
                  TAU = TAUQ( IJ )
                  CALL ZGEBS2D( ICTXT, 'Rowwise', ' ', 1, 1, TAU, 1 )
               ELSE
                  CALL ZGEBR2D( ICTXT, 'Rowwise', ' ', 1, 1, TAU, 1,
     $                          MYROW, IACOL )
               END IF
            END IF
            CALL PZSCAL( N-K, TAU, WORK( IPY ), 1, JWY, DESCWY,
     $                   DESCWY( 1 ) )
            CALL PZLACGV( N-K, WORK( IPY ), 1, JWY, DESCWY,
     $                    DESCWY( 1 ) )
            CALL PZCOPY( N-K, WORK( IPY ), 1, JWY, DESCWY, DESCWY( 1 ),
     $                   Y, IY+K-1, JY+K, DESCY, DESCY( 1 ) )
*
*           Update A(i,j+1:ja+n-1)
*
            CALL PZLACGV( N-K, A, I, J+1, DESCA, DESCA( 1 ) )
            CALL PZLACGV( K, A, I, JA, DESCA, DESCA( 1 ) )
            CALL PZGEMV( 'Conjugate transpose', K, N-K, -ONE, Y, IY,
     $                   JY+K, DESCY, A, I, JA, DESCA, DESCA( 1 ), ONE,
     $                   A, I, J+1, DESCA, DESCA( 1 ) )
            CALL PZLACGV( K, A, I, JA, DESCA, DESCA( 1 ) )
            CALL PZLACGV( K-1, X, IX+K-1, JX, DESCX, DESCX( 1 ) )
            CALL PZGEMV( 'Conjugate transpose', K-1, N-K, -ONE, A, IA,
     $                   J+1, DESCA, X, IX+K-1, JX, DESCX, DESCX( 1 ),
     $                   ONE, A, I, J+1, DESCA, DESCA( 1 ) )
            CALL PZLACGV( K-1, X, IX+K-1, JX, DESCX, DESCX( 1 ) )
            CALL PZELSET( A, I, J, DESCA, D( IJ ) )
*
*           Generate reflection P(i) to annihilate A(i,j+2:ja+n-1)
*
            IJ = MIN( II+K-1, MP )
            CALL PZLARFG( N-K, E( IJ ), I, J+1, A, I,
     $                    MIN( J+2, N+JA-1 ), DESCA, DESCA( 1 ), TAUP )
            CALL PZELSET( A, I, J+1, DESCA, ONE )
*
*           Compute X(I+1:IA+M-1,J)
*
            CALL PZGEMV( 'No transpose', M-K, N-K, ONE, A, I+1, J+1,
     $                   DESCA, A, I, J+1, DESCA, DESCA( 1 ), ZERO, X,
     $                   IX+K, JX+K-1, DESCX, 1 )
            CALL PZGEMV( 'No transpose', K, N-K, ONE, Y, IY, JY+K,
     $                   DESCY, A, I, J+1, DESCA, DESCA( 1 ), ZERO,
     $                   WORK, IW, 1, DESCW, 1 )
            CALL PZGEMV( 'No transpose', M-K, K, -ONE, A, I+1, JA,
     $                   DESCA, WORK, IW, 1, DESCW, 1, ONE, X, IX+K,
     $                   JX+K-1, DESCX, 1 )
            CALL PZGEMV( 'No transpose', K-1, N-K, ONE, A, IA, J+1,
     $                   DESCA, A, I, J+1, DESCA, DESCA( 1 ), ZERO,
     $                   WORK, IW, 1, DESCW, 1 )
            CALL PZGEMV( 'No transpose', M-K, K-1, -ONE, X, IX+K, JX,
     $                   DESCX, WORK, IW, 1, DESCW, 1, ONE, X, IX+K,
     $                   JX+K-1, DESCX, 1 )
*
            IF( MYCOL.EQ.IACOL ) THEN
               IF( MYROW.EQ.IAROW ) THEN
                  TAU = TAUP( IJ )
                  CALL ZGEBS2D( ICTXT, 'Columnwise', ' ', 1, 1, TAU, 1 )
               ELSE
                  CALL ZGEBR2D( ICTXT, 'Columnwise', ' ', 1, 1, TAU, 1,
     $                          IAROW, MYCOL )
               END IF
            END IF
            CALL PZSCAL( M-K, TAU, X, IX+K, JX+K-1, DESCX, 1 )
            CALL PZLACGV( N-K, A, I, J+1, DESCA, DESCA( 1 ) )
   10    CONTINUE
*
      ELSE
*
*        Reduce to lower bidiagonal form
*
         DO 20 K = 1, NB
            I = IA + K - 1
            J = JA + K - 1
            JWY = IW + K
*
*           Update A(i,j:ja+n-1)
*
            CALL PZLACGV( N-K+1, A, I, J, DESCA, DESCA( 1 ) )
            IF( K.GT.1 ) THEN
               CALL PZLACGV( K-1, A, I, JA, DESCA, DESCA( 1 ) )
               CALL PZGEMV( 'Conjugate transpose', K-1, N-K+1, -ONE, Y,
     $                      IY, JY+K-1, DESCY, A, I, JA, DESCA,
     $                      DESCA( 1 ), ONE, A, I, J, DESCA,
     $                      DESCA( 1 ) )
               CALL PZLACGV( K-1, A, I, JA, DESCA, DESCA( 1 ) )
               CALL PZLACGV( K-1, X, IX+K-1, JX, DESCX, DESCX( 1 ) )
               CALL PZGEMV( 'Conjugate transpose', K-1, N-K+1, -ONE, A,
     $                      IA, J, DESCA, X, IX+K-1, JX, DESCX,
     $                      DESCX( 1 ), ONE, A, I, J, DESCA,
     $                      DESCA( 1 ) )
               CALL PZLACGV( K-1, X, IX+K-1, JX, DESCX, DESCX( 1 ) )
               CALL PZELSET( A, I, J-1, DESCA, E( IJ ) )
            END IF
*
*           Generate reflection P(i) to annihilate A(i,j+1:ja+n-1)
*
            IJ = MIN( II+K-1, MP )
            CALL PZLARFG( N-K+1, D( IJ ), I, J, A, I, J+1, DESCA,
     $                    DESCA( 1 ), TAUP )
            CALL PZELSET( A, I, J, DESCA, ONE )
*
*           Compute X(i+1:ia+m-1,j)
*
            CALL PZGEMV( 'No transpose', M-K, N-K+1, ONE, A, I+1, J,
     $                   DESCA, A, I, J, DESCA, DESCA( 1 ), ZERO, X,
     $                   IX+K, JX+K-1, DESCX, 1 )
            CALL PZGEMV( 'No transpose', K-1, N-K+1, ONE, Y, IY, JY+K-1,
     $                   DESCY, A, I, J, DESCA, DESCA( 1 ), ZERO,
     $                   WORK, IW, 1, DESCW, 1 )
            CALL PZGEMV( 'No transpose', M-K, K-1, -ONE, A, I+1, JA,
     $                   DESCA, WORK, IW, 1, DESCW, 1, ONE, X, IX+K,
     $                   JX+K-1, DESCX, 1 )
            CALL PZGEMV( 'No transpose', K-1, N-K+1, ONE, A, IA, J,
     $                   DESCA, A, I, J, DESCA, DESCA( 1 ), ZERO,
     $                   WORK, IW, 1, DESCW, 1 )
            CALL PZGEMV( 'No transpose', M-K, K-1, -ONE, X, IX+K, JX,
     $                   DESCX, WORK, IW, 1, DESCW, 1, ONE, X, IX+K,
     $                   JX+K-1, DESCX, 1 )
*
            IF( MYCOL.EQ.IACOL ) THEN
               IF( MYROW.EQ.IAROW ) THEN
                  TAU = TAUP( IJ )
                  CALL ZGEBS2D( ICTXT, 'Columnwise', ' ', 1, 1, TAU, 1 )
               ELSE
                  CALL ZGEBR2D( ICTXT, 'Columnwise', ' ', 1, 1, TAU, 1,
     $                          IAROW, MYCOL )
               END IF
            END IF
            CALL PZSCAL( M-K, TAU, X, IX+K, JX+K-1, DESCX, 1 )
            CALL PZLACGV( N-K+1, A, I, J, DESCA, DESCA( 1 ) )
*
*           Update A(i+1:ia+m-1,j)
*
            CALL PZGEMV( 'No transpose', M-K, K-1, -ONE, A, I+1, JA,
     $                   DESCA, Y, IY, JY+K-1, DESCY, 1, ONE, A, I+1, J,
     $                   DESCA, 1 )
            CALL PZGEMV( 'No transpose', M-K, K, -ONE, X, IX+K, JX,
     $                   DESCX, A, IA, J, DESCA, 1, ONE, A, I+1, J,
     $                   DESCA, 1 )
            CALL PZELSET( A, I, J, DESCA, D( IJ ) )
*
*           Generate reflection Q(i) to annihilate A(i+2:ia+m-1,j)
*
            IJ = MIN( JJ+K-1, NQ )
            CALL PZLARFG( M-K, E( IJ ), I+1, J, A, MIN( I+2, M+IA-1 ),
     $                    J, DESCA, 1, TAUQ )
            CALL PZELSET( A, I+1, J, DESCA, ONE )
*
*           Compute Y(ia+i:ia+n-1,j)
*
            CALL PZGEMV( 'Conjugate transpose', M-K, N-K, ONE, A, I+1,
     $                   J+1, DESCA, A, I+1, J, DESCA, 1, ZERO,
     $                   WORK( IPY ), 1, JWY, DESCWY, DESCWY( 1 ) )
            CALL PZGEMV( 'Conjugate transpose', M-K, K-1, ONE, A, I+1,
     $                   JA, DESCA, A, I+1, J, DESCA, 1, ZERO, WORK, IW,
     $                   1, DESCW, 1 )
            CALL PZGEMV( 'Conjugate transpose', K-1, N-K, -ONE, Y, IY,
     $                   JY+K, DESCY, WORK, IW, 1, DESCW, 1, ONE,
     $                   WORK( IPY ), 1, JWY, DESCWY, DESCWY( 1 ) )
            CALL PZGEMV( 'Conjugate transpose', M-K, K, ONE, X, IX+K,
     $                   JX, DESCX, A, I+1, J, DESCA, 1, ZERO, WORK, IW,
     $                   1, DESCW, 1 )
            CALL PZGEMV( 'Conjugate transpose', K, N-K, -ONE, A, IA,
     $                   J+1, DESCA, WORK, IW, 1, DESCW, 1, ONE,
     $                   WORK( IPY ), 1, JWY, DESCWY, DESCWY( 1 ) )
*
            IF( MYROW.EQ.IAROW ) THEN
               IF( MYCOL.EQ.IACOL ) THEN
                  TAU = TAUQ( IJ )
                  CALL ZGEBS2D( ICTXT, 'Rowwise', ' ', 1, 1, TAU, 1 )
               ELSE
                  CALL ZGEBR2D( ICTXT, 'Rowwise', ' ', 1, 1, TAU, 1,
     $                          MYROW, IACOL )
               END IF
            END IF
            CALL PZSCAL( N-K, TAU, WORK( IPY ), 1, JWY, DESCWY,
     $                   DESCWY( 1 ) )
            CALL PZLACGV( N-K, WORK( IPY ), 1, JWY, DESCWY,
     $                    DESCWY( 1 ) )
            CALL PZCOPY( N-K, WORK( IPY ), 1, JWY, DESCWY, DESCWY( 1 ),
     $                   Y, IY+K-1, JY+K, DESCY, DESCY( 1 ) )
   20    CONTINUE
      END IF
*
      RETURN
*
*     End of PZLABRD
*
      END
