      SUBROUTINE PDLAGSY( N, K, D, A, IA, JA, DESCA, ISEED, ORDER, WORK,
     $                    LWORK, INFO )
*
*
*
*  -- ScaLAPACK routine (version 1.0) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     February 28, 1995
*
*     .. Scalar Arguments ..
      INTEGER            IA, INFO, JA, K, LWORK, N, ORDER
*     ..
*     .. Array Arguments ..
      INTEGER            DESCA( 8 ), ISEED( 4 )
      DOUBLE PRECISION   A( * ), D( * ), WORK( * )
*     ..
*
*  Notes
*  =====
*
*
*  A description vector is associated with each 2D block-cyclicly dis-
*  tributed matrix.  This vector stores the information required to
*  establish the mapping between a matrix entry and its corresponding
*  process and memory location.
*
*  In the following comments, the character _ should be read as
*  "of the distributed matrix".  Let A be a generic term for any 2D
*  block cyclicly distributed matrix.  Its description vector is DESCA:
*
*  NOTATION        STORED IN  EXPLANATION
*  --------------- ---------- ------------------------------------------
*  M_A    (global) DESCA( 1 ) The number of rows in the distributed
*                             matrix.
*  N_A    (global) DESCA( 2 ) The number of columns in the distributed
*                             matrix.
*  MB_A   (global) DESCA( 3 ) The blocking factor used to distribute
*                             the rows of the matrix.
*  NB_A   (global) DESCA( 4 ) The blocking factor used to distribute
*                             the columns of the matrix.
*  RSRC_A (global) DESCA( 5 ) The process row over which the first row
*                             of the matrix is distributed.
*  CSRC_A (global) DESCA( 6 ) The process column over which the first
*                             column of the matrix is distributed.
*  CTXT_A (global) DESCA( 7 ) The BLACS context handle, indicating the
*                             global context of the operation on the
*                             matrix. The context itself is global.
*  LLD_A  (local)  DESCA( 8 ) The leading dimension of the local array
*                             storing the local blocks of the distri-
*                             buted matrix A. LLD_A >= MAX(1,LOCp(M_A)).
*
*  Let K be the number of rows or columns of a distributed matrix,
*  and assume that its process grid has dimension p x q.
*  LOCp( K ) denotes the number of elements of K that a process
*  would receive if K were distributed over the p processes of its
*  process column.
*  Similarly, LOCq( K ) denotes the number of elements of K that a
*  process would receive if K were distributed over the q processes of
*  its process row.
*  The values of LOCp() and LOCq() may be determined via a call to the
*  ScaLAPACK tool function, NUMROC:
*          LOCp( M ) = NUMROC( M, MB_A, MYROW, RSRC_A, NPROW ),
*          LOCq( N ) = NUMROC( N, NB_A, MYCOL, CSRC_A, NPCOL ).
*
*  Purpose
*  =======
*
*  PDLAGSY generates a real symmetric matrix A, by pre- and post-
*  multiplying a real diagonal matrix D with a random orthogonal matrix:
*  A = U*D*U'.
*
*  This is just a quick implementation which will be replaced in the
*  future.  The random orthogonal matrix is computed by creating a
*  random matrix and running QR on it.  This requires vastly more
*  computation than necessary, but not significantly more communication
*  than is used in the rest of this rouinte, and hence is not that much
*  slower than an efficient solution.
*
*  Arguments
*  =========
*
*  N       (global input) INTEGER
*          The size of the matrix A.  N >= 0.
*
*  K       (global input) INTEGER
*          The number of nonzero subdiagonals within the band of A.
*          0 <= K <= N-1.
*          ### K must be 0 or N-1, 0 < K < N-1 is not supported yet.
*
*  D       (global input) DOUBLE PRECISION array, dimension (N)
*          The diagonal elements of the diagonal matrix D.
*
*  A       (local output) DOUBLE PRECISION array
*          Global dimension (N, N), local dimension (NP, NQ)
*          The generated n by n symmetric matrix A (the full matrix is
*          stored).
*
*
*  IA      (global input) INTEGER
*          The global row index of the submatrix of the distributed
*          matrix A to operate on.
*
*
*  JA      (global input) INTEGER
*          The global column index of the submatrix of the distributed
*          matrix A to operate on.
*
*
*  DESCA   (global and local input) INTEGER array of dimension 8
*          The array descriptor for the distributed matrix A.
*
*  ISEED   (global input/output) INTEGER array, dimension (4)
*          On entry, the seed of the random number generator; the array
*          elements must be between 0 and 4095, and ISEED(4) must be
*          odd.
*          On exit, the seed is updated and will remain identical on
*          all processes in the context.
*
*  ORDER   (global input) INTEGER
*          Number of reflectors in the matrix Q
*          At present, ORDER .NE. N is not supported
*
*  WORK    (local workspace) DOUBLE PRECISION array, dimension (LWORK)
*
*  LWORK   (local input) INTEGER dimension of WORK
*        LWORK >= (LDAA+1)*MAX( 1, NQ ) + 200 + MAX( SIZEMQRLEFT,
*             SIZEMQRRIGHT, SIZEQRF )
*        Where:
*         LDAA = DESCA( 8 )
*         MB_A = DESCA( 3 )
*         NB_A = DESCA( 4 )
*         RSRC_A = DESCA( 5 )
*         CSRC_A = DESCA( 6 )
*         LCM = ILCM( NPROW, NPCOL )
*         LCMQ = LCM / NPCOL
*         IROFFA = MOD( IA-1, MB_A )
*         ICOFFA = MOD( JA-1, NB_A )
*         IAROW = INDXG2P( IA, MB_A, MYROW, RSRC_A, NPROW )
*         IACOL = INDXG2P( JA, NB_A, MYCOL, CSRC_A, NPCOL )
*         NP = NUMROC( N+IROFFA, MB_A, MYROW, IAROW, NPROW )
*         NQ = NUMROC( N+ICOFFA, NB_A, MYCOL, IACOL, NPCOL )
*         SIZEMQRLEFT = MAX( (MB_A*(MB_A-1))/2, ( NP + NQ ) * MB_A )
*     $        + ( NP + NB_A ) * NB_A
*         SIZEMQRRIGHT =  MAX( (NB_A*(NB_A-1))/2,
*     $        ( NQ + MAX( NP + NUMROC( NUMROC( N+ICOFFA,
*     $        NB_A, 0, 0, NPCOL ), NB_A, 0, 0, LCMQ ), NP ))*NB_A )
*     $        NB_A * NB_A
*         SIZEQRF = NB_A*NP + MB_A*NQ + NB_A*NB_A
*
*
*  INFO    (local output) INTEGER
*
*          = 0:  successful exit
*          < 0:  If the i-th argument is an array and the j-entry had
*                an illegal value, then INFO = -(i*100+j), if the i-th
*                argument is a scalar and had an illegal value, then
*                INFO = -i.
*
*
*     .. Parameters ..
      DOUBLE PRECISION   ZERO
      PARAMETER          ( ZERO = 0.0D+0 )
*     ..
*     .. Local Scalars ..
*
*
*
      INTEGER            CSRC_A, I, IACOL, IAROW, ICOFFA, II, IIROW,
     $                   IROFFA, JJCOL, LCM, LCMQ, LDAA, LII, LIII, LJJ,
     $                   LJJJ, LWMIN, MAXI, MB_A, MYCOL, MYROW, NB_A,
     $                   NP, NPCOL, NPROW, NQ, PTR2AA, PTR2TAU,
     $                   PTR2WORK, RSRC_A, SIZEMQRLEFT, SIZEMQRRIGHT,
     $                   SIZEQRF
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GRIDINFO, CHK1MAT, DLASET, PDGEQRF,
     $                   PDMATGEN, PDORMQR, PXERBLA
*     ..
*     .. External Functions ..
      INTEGER            ILCM, INDXG2P, NUMROC
      EXTERNAL           ILCM, INDXG2P, NUMROC
*     ..
*     .. Intrinsic Functions ..
*
*
*
      INTRINSIC          MAX, MIN, MOD
*     ..
*     .. Executable Statements ..
*
*     Initialize grid information
*
      CALL BLACS_GRIDINFO( DESCA( 7 ), NPROW, NPCOL, MYROW, MYCOL )
*
*     Check LWORK
*
      INFO = 0
      IF( NPROW.EQ.-1 ) THEN
         INFO = -607
      ELSE
         CALL CHK1MAT( N, 1, N, 1, IA, JA, DESCA, 7, INFO )
      END IF
*
      LDAA = DESCA( 8 )
      MB_A = DESCA( 3 )
      NB_A = DESCA( 4 )
      RSRC_A = DESCA( 5 )
      CSRC_A = DESCA( 6 )
      LCM = ILCM( NPROW, NPCOL )
      LCMQ = LCM / NPCOL
      IROFFA = MOD( IA-1, MB_A )
      ICOFFA = MOD( JA-1, NB_A )
      IAROW = INDXG2P( IA, MB_A, MYROW, RSRC_A, NPROW )
      IACOL = INDXG2P( JA, NB_A, MYCOL, CSRC_A, NPCOL )
      NP = NUMROC( N+IROFFA, MB_A, MYROW, IAROW, NPROW )
      NQ = NUMROC( N+ICOFFA, NB_A, MYCOL, IACOL, NPCOL )
*
      SIZEMQRLEFT = MAX( ( MB_A*( MB_A-1 ) ) / 2, ( NP+NQ )*MB_A ) +
     $              ( NP+NB_A )*NB_A
      SIZEMQRRIGHT = MAX( ( NB_A*( NB_A-1 ) ) / 2,
     $               ( NQ+MAX( NP+NUMROC( NUMROC( N+ICOFFA, NB_A, 0, 0,
     $               NPCOL ), NB_A, 0, 0, LCMQ ), NP ) )*NB_A ) +
     $               NB_A*NB_A
      SIZEQRF = NB_A*NP + MB_A*NQ + NB_A*NB_A
*
      LWMIN = ( LDAA+1 )*MAX( 1, NQ ) + 200 +
     $        MAX( SIZEMQRLEFT, SIZEMQRRIGHT, SIZEQRF )
*
*     Test the input arguments
*
      IF( INFO.EQ.0 ) THEN
         IF( K.LT.0 .OR. K.GT.N-1 ) THEN
            INFO = -3
         ELSE IF( N.NE.ORDER ) THEN
            INFO = -9
         ELSE IF( LWORK.LT.LWMIN ) THEN
            INFO = -11
         END IF
      END IF
      IF( INFO.LT.0 ) THEN
         CALL PXERBLA( DESCA( 7 ), 'PDLAGSY', -INFO )
         RETURN
      END IF
*
      PTR2AA = 1
      PTR2TAU = PTR2AA + LDAA*MAX( 1, NQ ) + 100
      PTR2WORK = PTR2TAU + MAX( 1, NQ ) + 100
*
      IF( K.NE.0 ) THEN
         CALL DLASET( 'A', LDAA, NQ, ZERO, ZERO, WORK( PTR2AA ), LDAA )
*
*
*        Build a random matrix
*
*
         CALL PDMATGEN( DESCA( 7 ), 'N', 'N', N, ORDER, DESCA( 3 ),
     $                  DESCA( 4 ), WORK( PTR2AA ), DESCA( 8 ),
     $                  DESCA( 5 ), DESCA( 6 ), ISEED, 0, NP, 0, NQ,
     $                  MYROW, MYCOL, NPROW, NPCOL )
         CALL PDGEQRF( N, ORDER, WORK( PTR2AA ), IA, JA, DESCA,
     $                 WORK( PTR2TAU ), WORK( PTR2WORK ), SIZEQRF,
     $                 INFO )
*
      END IF
*
*     Build a diagonal matrix A with the eigenvalues specified in D
*
      CALL DLASET( 'A', NP, NQ, ZERO, ZERO, A, DESCA( 8 ) )
*
      IIROW = 0
      JJCOL = 0
      LII = 1
      LJJ = 1
*
      DO 20 II = 1, N, DESCA( 3 )
         MAXI = MIN( N, II+DESCA( 3 )-1 )
         IF( ( MYROW.EQ.IIROW ) .AND. ( MYCOL.EQ.JJCOL ) ) THEN
            LIII = LII
            LJJJ = LJJ
            DO 10 I = II, MAXI
               A( LIII+( LJJJ-1 )*DESCA( 8 ) ) = D( I )
               LIII = LIII + 1
               LJJJ = LJJJ + 1
   10       CONTINUE
         END IF
         IF( MYROW.EQ.IIROW )
     $      LII = LII + DESCA( 3 )
         IF( MYCOL.EQ.JJCOL )
     $      LJJ = LJJ + DESCA( 3 )
         IIROW = MOD( IIROW+1, NPROW )
         JJCOL = MOD( JJCOL+1, NPCOL )
   20 CONTINUE
*
*     A = Q * A
*
      IF( K.NE.0 ) THEN
*
         CALL PDORMQR( 'L', 'T', N, N, ORDER, WORK( PTR2AA ), IA, JA,
     $                 DESCA, WORK( PTR2TAU ), A, IA, JA, DESCA,
     $                 WORK( PTR2WORK ), SIZEMQRLEFT, INFO )
*
*
*        A = A * Q'
*
*
         CALL PDORMQR( 'R', 'N', N, N, ORDER, WORK( PTR2AA ), IA, JA,
     $                 DESCA, WORK( PTR2TAU ), A, IA, JA, DESCA,
     $                 WORK( PTR2WORK ), SIZEMQRRIGHT, INFO )
*
      END IF
*
*     End of PDLAGSY
*
      END
