      PROGRAM PDSEPDRIVER
*
*
*  -- ScaLAPACK routine (version 1.0) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     February 28, 1995
*
*     Parallel Double precision Symmetric eigenproblem test driver
*
*     The user should modify TOTMEM to indicate the maximum amount of
*     memory in bytes her system has.  Remember to leave room in memory
*     for operating system, the BLACS buffer, etc.  INTSIZ and DBLSIZ
*     indicate the length in bytes on the given platform for an integer
*     and a double precision real.
*     For example, on our system with 8 MB of memory, TOTMEM=6500000
*     (leaves 1.5 MB for OS, code, BLACS buffer, etc), the length of a
*     DOUBLE is 8, and an integer takes up 4 bytes.  Some playing around
*     to discover what the maximum value you can set MEMSIZ to may be
*     required.
*     All arrays used by factorization and solve are allocated out of
*     big array called MEM.
*
*     The full tester requires approximately (5 n + 5 n^2/p + slop)
*     double precision words and 6*n integer words.
*     So, TOTMEM should be set to at least 1.1 * 8 * (5n + 5n^2/p)
*     IMEMSIZE should be set to 6n + 4
*
*     WHAT WE TEST
*     ============
*
*     This routine tests PDSYEVX, the expert driver for the parallel
*     symmetric eigenvalue problem.  We would like to cover all
*     possible combinations of:  matrix size, process configuration
*     (nprow and npcol), block size (nb), matrix type (??), range
*     of eigenvalue (all, by value, by position), sorting options,
*     and upper vs. lower storage.
*
*     We intend to provide two types of test input files, an
*     installation test and a thorough test.  The installation
*     test should complete in a minute or less on the CM5.  The thorough
*     test will be allowed to take up to an hour on the CM5.
*
*     We also intend that the reports be meaningful.  Our input file
*     will allow multiple requests where each request is a cross product
*     of the following sets:
*       matrix sizes:                     n
*       process configuration triples:  nprow, npcol, nb
*       matrix types:
*       eigenvalue requests:              all, by value, by position
*       storage (upper vs. lower):        uplo
*
*     Each request will specify either:
*       complete printouts (a printout for each test in cross product)
*       summary printout only (one printout for the whole cross product)
*
*     The summary printout feature will allow us to run a large number
*     of trivial, and hence fast, tests without creating a huge volume
*     of fairly meaningless output.
*
*     Allowing multiple requests will, in addition to allowing trivial
*     tests to be run with the summary printout feature, also allow
*     a few non trivial tests to be run with the full printout, and
*     it will allow a very few large tests to be performed.  Hence,
*     we imagine the input files for this test to look like:
*
*       A set of trivial tests (matrix sizes 0,1,2) with a full set
*         of options but only summary printing
*       A set of medium sized tests (matrix sizes 10-100) with
*         a full set of options and full printing
*       A set of large tests with only a few options and full printing
*
*     TERMS:
*       Request - means a set of tests, which is the cross product of
*       a set of specifications from the input file.
*       Test - one element in the cross product, i.e. a specific input
*       size and type, process configuration, etc.
*
*
*
*     .. Parameters ..
*
      INTEGER            MAXN
      PARAMETER          ( MAXN = 40000 )
      INTEGER            NIN
      PARAMETER          ( NIN = 11 )
      INTEGER            TOTMEM
      PARAMETER          ( TOTMEM = 2000000 )
      INTEGER            IMEMSIZ
      PARAMETER          ( IMEMSIZ = MAXN*6+4 )
      INTEGER            DBLSIZ
      PARAMETER          ( DBLSIZ = 8 )
      INTEGER            MEMSIZ
      PARAMETER          ( MEMSIZ = TOTMEM / DBLSIZ )
      INTEGER            NREQS
      PARAMETER          ( NREQS = 50 )
*     ..
*     .. Local Scalars ..
      CHARACTER*80       SUMMRY, USRINFO
      INTEGER            ALLRESULT, CONT, CONTEXT, I, IAM, ISIEEE,
     $                   MAXNODES, NOUT, NPROCS, REQNUM
*     ..
*     .. Local Arrays ..
*
      INTEGER            IMEM( IMEMSIZ ), ISEED( 4 ), RESULT( NREQS )
      DOUBLE PRECISION   MEM( MEMSIZ )
*     ..
*     .. External Functions ..
      DOUBLE PRECISION   DLAMCH
      EXTERNAL           DLAMCH
*     ..
*     .. External Subroutines ..
*
      EXTERNAL           BLACS_EXIT, BLACS_GET, BLACS_GRIDEXIT,
     $                   BLACS_GRIDINIT, BLACS_PINFO, BLACS_SETUP,
     $                   IGAMN2D, PDLACHKIEEE, PDLASNBT, PDSEPREQ
*     ..
*     .. Executable Statements ..
*
*     Get starting information
*
      CALL BLACS_PINFO( IAM, NPROCS )
      REQNUM = 0
      ALLRESULT = 0
*
*
      IF( IAM.EQ.0 ) THEN
*
*        Open file and skip data file header
*
         OPEN( UNIT = NIN, FILE = 'SEP.dat', STATUS = 'OLD' )
         READ( NIN, FMT = * )SUMMRY
         SUMMRY = ' '
*
*        Read in user-supplied info about machine type, compiler, etc.
*
         READ( NIN, FMT = 9993 )USRINFO
*
*        Read name and unit number for summary output file
*
         READ( NIN, FMT = * )SUMMRY
         READ( NIN, FMT = * )NOUT
         IF( NOUT.NE.0 .AND. NOUT.NE.6 )
     $      OPEN( NOUT, FILE = SUMMRY, STATUS = 'UNKNOWN' )
         READ( NIN, FMT = * )MAXNODES
      END IF
*
      IF( NPROCS.LT.1 ) THEN
         CALL BLACS_SETUP( IAM, MAXNODES )
         NPROCS = MAXNODES
      END IF
*
      CALL BLACS_GET( -1, 0, CONTEXT )
      CALL BLACS_GRIDINIT( CONTEXT, 'R', 1, NPROCS )
*
      CALL PDLASNBT( ISIEEE )
*
      IF( ( ISIEEE.NE.0 ) ) THEN
         IF( IAM.EQ.0 ) THEN
            WRITE( NOUT, FMT = 9992 )
            WRITE( NOUT, FMT = 9991 )
            WRITE( NOUT, FMT = 9990 )
            WRITE( NOUT, FMT = 9989 )
            WRITE( NOUT, FMT = 9988 )
            WRITE( NOUT, FMT = 9987 )
            WRITE( NOUT, FMT = 9986 )
            WRITE( NOUT, FMT = 9985 )
            WRITE( NOUT, FMT = 9984 )
         END IF
*
         CALL PDLACHKIEEE( ISIEEE, DLAMCH( 'O' ), DLAMCH( 'U' ) )
*
         CALL IGAMN2D( CONTEXT, 'a', ' ', 1, 1, ISIEEE, 1, 1, 1, -1, -1,
     $                 0 )
*
         IF( ISIEEE.EQ.0 ) THEN
            IF( IAM.EQ.0 ) THEN
               WRITE( NOUT, FMT = 9983 )
               WRITE( NOUT, FMT = 9982 )
               WRITE( NOUT, FMT = 9981 )
            END IF
            GO TO 30
         END IF
*
         IF( IAM.EQ.0 ) THEN
            WRITE( NOUT, FMT = 9980 )
         END IF
*
      END IF
      IF( IAM.EQ.0 ) THEN
         WRITE( NOUT, FMT = 9993 )
     $      'SCALAPACK Symmetric Eigendecomposition routines.'
         WRITE( NOUT, FMT = 9993 )USRINFO
         WRITE( NOUT, FMT = 9993 )' '
         WRITE( NOUT, FMT = 9993 )'Running tests of the parallel ' //
     $      'symmetric eigenvalue routine:  PDSYEVX.'
         WRITE( NOUT, FMT = 9993 )'The following scaled residual ' //
     $      'checks will be computed:'
         WRITE( NOUT, FMT = 9993 )' ||AQ - QL|| ' //
     $      '/ ((abstol + ||A|| * eps) * N)'
         WRITE( NOUT, FMT = 9993 )' ||Q^T*Q - I|| ' // '/ (N * eps)'
         WRITE( NOUT, FMT = 9993 )
         WRITE( NOUT, FMT = 9993 )'An explanation of the ' //
     $      'input/output parameters follows:'
         WRITE( NOUT, FMT = 9993 )'RESULT   : passed; or ' //
     $      'an indication of which eigen request test failed'
         WRITE( NOUT, FMT = 9993 )
     $      'N        : The number of rows and columns ' //
     $      'of the matrix A.'
         WRITE( NOUT, FMT = 9993 )
     $      'P        : The number of process rows.'
         WRITE( NOUT, FMT = 9993 )
     $      'Q        : The number of process columns.'
         WRITE( NOUT, FMT = 9993 )
     $      'NB       : The size of the square blocks' //
     $      ' the matrix A is split into.'
         WRITE( NOUT, FMT = 9993 )
     $      'THRESH   : If a residual value is less ' //
     $      'than THRESH, RESULT is flagged as PASSED.'
         WRITE( NOUT, FMT = 9993 )
     $      '         : the QTQ norm is allowed to exceed THRESH' //
     $      ' for those eigenvectors'
         WRITE( NOUT, FMT = 9993 )'         :  which could not be ' //
     $      'reorthogonalized for lack of workspace.'
         WRITE( NOUT, FMT = 9993 )
     $      'TYP      : matrix type (see pdseptst.f).'
         WRITE( NOUT, FMT = 9993 )
     $      'NT       : number of eigen request tests ' //
     $      '(see pdseptst.f).'
         WRITE( NOUT, FMT = 9993 )'CHK      : ||AQ - QL|| ' //
     $      '/ ((abstol + ||A|| * eps) * N)'
         WRITE( NOUT, FMT = 9993 )'QTQ      : ||Q^T*Q - I||/ (N * eps)'
         WRITE( NOUT, FMT = 9993 )
     $      'If NT>1, CHK and QTQ are the max over all ' //
     $      'eigen request tests'
         WRITE( NOUT, FMT = 9993 )' '
      END IF
*
   10 CONTINUE
      REQNUM = REQNUM + 1
*
      ISEED( 1 ) = 139
      ISEED( 2 ) = 1139
      ISEED( 3 ) = 2139
      ISEED( 4 ) = 3139
*
      CALL PDSEPREQ( NIN, RESULT( REQNUM ), MEM, MEMSIZ, IMEM, IMEMSIZ,
     $               6, ISEED, CONT )
      ALLRESULT = ALLRESULT + RESULT( REQNUM )
*
      IF( IAM.EQ.0 .AND. CONT.EQ.0 ) THEN
         IF( RESULT( REQNUM ).NE.0 ) THEN
            WRITE( NOUT, FMT = 9979 )REQNUM
         ELSE
            WRITE( NOUT, FMT = 9978 )REQNUM
         END IF
      END IF
*
      IF( CONT.EQ.0 )
     $   GO TO 10
*
      IF( IAM.EQ.0 ) THEN
         IF( ALLRESULT.NE.0 ) THEN
            DO 20 I = 1, REQNUM - 1
               IF( RESULT( I ).NE.0 ) THEN
                  WRITE( NOUT, FMT = 9979 )I
               ELSE
                  WRITE( NOUT, FMT = 9978 )I
               END IF
   20       CONTINUE
*
            WRITE( NOUT, FMT = 9996 )ALLRESULT
         END IF
         WRITE( NOUT, FMT = 9993 )REQNUM - ALLRESULT - 1
*
         IF( ALLRESULT.EQ.0 ) THEN
            WRITE( NOUT, FMT = 9995 )
         ELSE
            WRITE( NOUT, FMT = 9994 )
         END IF
      END IF
*
*     Uncomment this line on SUN systems to avoid the useless print out
*
*      CALL IEEE_FLAGS( 'clear', 'exception', 'underflow', '')
*
*
*
   30 CONTINUE
      IF( IAM.EQ.0 ) THEN
         CLOSE ( NIN )
         CLOSE ( NOUT )
      END IF
*
      CALL BLACS_GRIDEXIT( CONTEXT )
*
      CALL BLACS_EXIT( 0 )
      STOP
*
*     End of PDSEPDRIVER .
*
*
*
 9999 FORMAT( 'Test request passed' )
 9998 FORMAT( I5, ' test requests completed without errors' )
 9997 FORMAT( ' At least 1 test request was skipped due to input error'
     $       )
 9996 FORMAT( I5, ' test requests failed' )
 9995 FORMAT( 'TEST SUCCEEDED' )
 9994 FORMAT( 'TEST FAILED' )
*
 9993 FORMAT( A )
 9992 FORMAT( ' I am about to check to make sure that overflow' )
 9991 FORMAT( ' is handled in the ieee default manner.  If this' )
 9990 FORMAT( ' is the last output you see, you should assume' )
 9989 FORMAT( ' that overflow caused a floating point exception.' )
 9988 FORMAT( ' In that case, we recommend that you add -DNO_IEEE' )
 9987 FORMAT( ' to the CDEFS line in SLmake.inc.' )
 9986 FORMAT( ' Alternatively, you could set CDEFS in SLmake.inc ' )
 9985 FORMAT( ' to enable the default ieee behaviour, However, this' )
 9984 FORMAT( ' may result in good or very bad performance.' )
 9983 FORMAT( ' Either signed zeroes or signed infinities ' )
 9982 FORMAT( ' work incorrectly or your system.  Change your' )
 9981 FORMAT( ' SLmake.inc as suggested above.' )
*
 9980 FORMAT( ' Your system appears to handle ieee overflow.' )
 9979 FORMAT( ' TEST REQUEST ', I5, ' failed' )
 9978 FORMAT( ' TEST REQUEST ', I5, ' passed' )
*
*
      END
