      PROGRAM SAMPLE_PDSYEVX_CALL
*
*
*  -- ScaLAPACK example routine (version 1.0)
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     December, 1994
*
*
*     This routine contains a sample call to PDSYEVX.
*     When compiled and run, it produces output which can be
*     pasted directly into matlab.
*
*
*
*     .. Parameters ..
      INTEGER            LWORK, MAXN, LIWORK
      DOUBLE PRECISION   ZERO
      PARAMETER          ( LWORK = 100000, MAXN = 100, LIWORK = 5000,
     $                   ZERO = 0.0D0 )
      INTEGER            LDA
      DOUBLE PRECISION   MONE
      INTEGER            MAXPROCS
      PARAMETER          ( LDA = MAXN, MONE = -1.0D0, MAXPROCS = 512 )
*     ..
*     .. Local Scalars ..
      INTEGER            CONTEXT, I, IAM, INFO, M, MYCOL, MYROW, N, NB,
     $                   NPCOL, NPROCS, NPROW, NZ
*     ..
*     .. Local Arrays ..
      INTEGER            DESCA( 8 ), DESCZ( 8 ), ICLUSTR( MAXPROCS*2 ),
     $                   IFAIL( MAXN ), IWORK( LIWORK )
      DOUBLE PRECISION   A( LDA, LDA ), GAP( MAXPROCS ), W( MAXN ),
     $                   WORK( LWORK ), Z( LDA, LDA )
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_EXIT, BLACS_GET, BLACS_GRIDEXIT,
     $                   BLACS_GRIDINFO, BLACS_GRIDINIT, BLACS_PINFO,
     $                   BLACS_SETUP, PDLAMODHILB, PDLAPRNT, PDSYEVX
*     ..
*     .. Executable Statements ..
*
*
*     Set up the problem
*
      N = 4
      NB = 1
      NPROW = 2
      NPCOL = 1
*
*
*     Initialize the BLACS
*
      CALL BLACS_PINFO( IAM, NPROCS )
      IF( ( NPROCS.LT.1 ) ) THEN
         CALL BLACS_SETUP( IAM, NPROW*NPCOL )
      END IF
*
*
*     Initialize a single BLACS context
*
      CALL BLACS_GET( -1, 0, CONTEXT )
      CALL BLACS_GRIDINIT( CONTEXT, 'R', NPROW, NPCOL )
      CALL BLACS_GRIDINFO( CONTEXT, NPROW, NPCOL, MYROW, MYCOL )
*
*     Bail out if this process is not a part of this context.
*
      IF( MYROW.EQ.-1 )
     $   GO TO 20
*
*
*     These are basic array descriptors
*
      DESCA( 1 ) = N
      DESCA( 2 ) = N
      DESCA( 3 ) = NB
      DESCA( 4 ) = NB
      DESCA( 5 ) = 0
      DESCA( 6 ) = 0
      DESCA( 7 ) = CONTEXT
      DESCA( 8 ) = LDA
*
      DESCZ( 1 ) = N
      DESCZ( 2 ) = N
      DESCZ( 3 ) = NB
      DESCZ( 4 ) = NB
      DESCZ( 5 ) = 0
      DESCZ( 6 ) = 0
      DESCZ( 7 ) = CONTEXT
      DESCZ( 8 ) = LDA
*
*
*     Build a matrix that you can create with
*     a one line matlab command:  hilb(n) + diag([1:-1/n:1/n])
*
      CALL PDLAMODHILB( N, A, 1, 1, DESCA, INFO )
*
*
*     Uncomment this line to see the matrix printed out.
*
*      CALL PDLAPRNT( N, N, A, 1, 1, DESCA, 0, 0, 'A', 6, WORK )
*
*
*     Ask PDSYEVX to compute the entire eigendecomposition
*
      CALL PDSYEVX( 'V', 'A', 'U', N, A, 1, 1, DESCA, ZERO, ZERO, 13,
     $              -13, MONE, M, NZ, W, MONE, Z, 1, 1, DESCZ, WORK,
     $              LWORK, IWORK, LIWORK, IFAIL, ICLUSTR, GAP, INFO )
*
*
*     Print out the eigenvectors
*
      CALL PDLAPRNT( N, N, Z, 1, 1, DESCZ, 0, 0, 'Z', 6, WORK )
*
*
*     Print out matlab code which will check the residual
*
      IF( MYROW.EQ.0 .AND. MYCOL.EQ.0 ) THEN
         PRINT *, ' N =', N
         PRINT *, ' A =  hilb(N) + diag([1:-1/N:1/N])'
         DO 10 I = 1, N
            PRINT *, ' W(', I, ')=', W( I ), ';'
   10    CONTINUE
         PRINT *, ' ortho = A - Z * diag(W) * Z'' '
         PRINT *, ' resid = A * Z - Z * diag(W)'
         PRINT *, ' norm(ortho)'
         PRINT *, ' norm(resid)'
      END IF
*
      CALL BLACS_GRIDEXIT( CONTEXT )
*
   20 CONTINUE
*
      CALL BLACS_EXIT( 0 )
*
*
*     Uncomment this line on SUN systems to avoid the useless print out
*
*      CALL IEEE_FLAGS( 'clear', 'exception', 'underflow', '')
*
*
 9999 FORMAT( 'W=diag([', 4D16.12, ']);' )
*
      STOP
      END
*
      SUBROUTINE PDLAMODHILB( N, A, IA, JA, DESCA, INFO )
*
*  -- ScaLAPACK example routine (version 1.0)
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     December, 1994
*
*
*
*
*     .. Parameters ..
      DOUBLE PRECISION   ONE
      PARAMETER          ( ONE = 1.0D0 )
*     ..
*     .. Scalar Arguments ..
      INTEGER            IA, INFO, JA, N
*     ..
*     .. Array Arguments ..
      INTEGER            DESCA( 8 )
      DOUBLE PRECISION   A( * )
*     ..
*     .. Local Scalars ..
      INTEGER            I, J, MYCOL, MYROW, NPCOL, NPROW
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GRIDINFO, PDELSET
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          DBLE
*     ..
*     .. Executable Statements ..
*
*
*     The matlab code looks like:
*         hilb(n) + diag([1:-1/n:1/n])
*
*
      INFO = 0
*
      CALL BLACS_GRIDINFO( DESCA( 7 ), NPROW, NPCOL, MYROW, MYCOL )
*
*
      IF( IA.NE.1 ) THEN
         INFO = -3
      ELSE IF( JA.NE.1 ) THEN
         INFO = -4
      END IF
*
      DO 20 J = 1, N
         DO 10 I = 1, N
            IF( I.EQ.J ) THEN
               CALL PDELSET( A, I, J, DESCA,
     $                       ( DBLE( N-I+1 ) ) / DBLE( N )+ONE /
     $                       ( DBLE( I+J )-ONE ) )
            ELSE
               CALL PDELSET( A, I, J, DESCA, ONE / ( DBLE( I+J )-ONE ) )
            END IF
   10    CONTINUE
   20 CONTINUE
*
*
      RETURN
      END
